@@ -1008,8 +1008,12 @@ loads(PyObject* self, PyObject* args, PyObject* kwargs)
         return NULL;
 
     if (objectHook && !PyCallable_Check(objectHook)) {
-        PyErr_SetString(PyExc_TypeError, "object_hook is not callable");
-        return NULL;
+        if (objectHook == Py_None)
+            objectHook = NULL;
+        else {
+            PyErr_SetString(PyExc_TypeError, "object_hook is not callable");
+            return NULL;
+        }
     }
 
     Py_ssize_t jsonStrLen;
@@ -1278,11 +1282,16 @@ do_decode(PyObject* decoder, const char* jsonStr, Py_ssize_t jsonStrLen,
             PyObject* etraceback;
             PyErr_Fetch(&etype, &evalue, &etraceback);
 
+            // Try to add the offset in the error message if the exception
+            // value is a string.  Otherwise, use the original exception since
+            // we can't be sure the exception type takes a single string.
             const char* emsg = msg;
-            if (PyUnicode_Check(evalue))
+            if (PyUnicode_Check(evalue)) {
                 emsg = PyUnicode_AsUTF8(evalue);
-
-            PyErr_Format(etype, fmt, offset, emsg);
+                PyErr_Format(etype, fmt, offset, emsg);
+            }
+            else
+                PyErr_Restore(etype, evalue, etraceback);
         }
         else
             PyErr_Format(PyExc_ValueError, fmt, offset, msg);
@@ -1675,6 +1684,8 @@ dumps_internal(
         else if (PyUnicode_Check(object)) {
             Py_ssize_t l;
             char* s = PyUnicode_AsUTF8AndSize(object, &l);
+            if (s == NULL)
+                return NULL;
             writer->String(s, l);
         }
         else if (PyList_Check(object)) {
@@ -1712,6 +1723,8 @@ dumps_internal(
                     if (PyUnicode_Check(key)) {
                         Py_ssize_t l;
                         char* key_str = PyUnicode_AsUTF8AndSize(key, &l);
+                        if (key_str == NULL)
+                            goto error;
                         stack.push_back(WriterContext(NULL, 0, item, false, nextLevel));
                         stack.push_back(WriterContext(key_str, l, NULL, false, nextLevel));
                     }
@@ -1728,6 +1741,8 @@ dumps_internal(
                     if (PyUnicode_Check(key)) {
                         Py_ssize_t l;
                         char* key_str = PyUnicode_AsUTF8AndSize(key, &l);
+                        if (key_str == NULL)
+                            goto error;
                         items.push_back(DictItem(key_str, l, item));
                     }
                     else if (!skipKeys) {
@@ -2124,8 +2139,12 @@ dumps(PyObject* self, PyObject* args, PyObject* kwargs)
         return NULL;
 
     if (defaultFn && !PyCallable_Check(defaultFn)) {
-        PyErr_SetString(PyExc_TypeError, "default must be a callable");
-        return NULL;
+        if (defaultFn == Py_None)
+            defaultFn = NULL;
+        else {
+            PyErr_SetString(PyExc_TypeError, "default must be a callable");
+            return NULL;
+        }
     }
 
     if (indent && indent != Py_None) {
